'use strict';

function _possibleConstructorReturn(self, call) { if (!self) { throw new ReferenceError("this hasn't been initialised - super() hasn't been called"); } return call && (typeof call === "object" || typeof call === "function") ? call : self; }

function _inherits(subClass, superClass) { if (typeof superClass !== "function" && superClass !== null) { throw new TypeError("Super expression must either be null or a function, not " + typeof superClass); } subClass.prototype = Object.create(superClass && superClass.prototype, { constructor: { value: subClass, enumerable: false, writable: true, configurable: true } }); if (superClass) Object.setPrototypeOf ? Object.setPrototypeOf(subClass, superClass) : subClass.__proto__ = superClass; }

function _classCallCheck(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

/**
 * Licensed Materials - Property of IBM
 * IBM Cognos Products: BI Cloud (C) Copyright IBM Corp. 2020
 * US Government Users Restricted Rights - Use, duplication or disclosure restricted by GSA ADP Schedule Contract with IBM Corp.
 */
define([], function () {
	var msg_average = 'Average';
	var msg_sum = 'Sum';
	/**
  * Vida works with the concept of decorations. This is required to implement selections/highlighting.
  */

	var Decorated = function () {
		function Decorated() {
			_classCallCheck(this, Decorated);
		}

		// IDecorated

		/**
   * Check for existence of a decoration.
   * @param {string} _name The name of the decoration.
   * @returns {Boolean} True if the decoration is present on the object, false otherwise.
   */
		Decorated.prototype.hasDecoration = function hasDecoration(_name) {
			return this.hasOwnProperty(_name);
		};

		/**
   * Get the value of a decoration.
   * @param {String} _name The name of the decoration.
   * @returns {any} The value of the decoration.
   */


		Decorated.prototype.getDecoration = function getDecoration(_name) {
			return this[_name];
		};

		return Decorated;
	}();

	/**
 	 * A DataItem is the abstract base for categorical or continuous data items in VIPR.
 	 * A DataItem is much like a Column, but categorical data items in VIPR allow stacking.
 	 * (This is not supported in this implementation).
 	 */


	var DataItem = function (_Decorated) {
		_inherits(DataItem, _Decorated);

		function DataItem(_type, _caption, _uniqueName, _kind, _columnIndex) {
			_classCallCheck(this, DataItem);

			var _this = _possibleConstructorReturn(this, _Decorated.call(this));

			Object.defineProperties(_this, {
				type: {
					enumerable: true,
					value: _type
				},
				caption: {
					enumerable: true,
					value: _caption
				},
				uniqueName: {
					enumerable: true,
					value: _uniqueName
				},
				// Not used by ViPR
				kind: {
					enumerable: true,
					value: _kind
				},
				columnIndex: {
					enumerable: true,
					value: _columnIndex
				}
			});
			return _this;
		}

		DataItem.prototype.toString = function toString() {
			return this.label;
		};

		// IDataItem

		/**
   * Report the type of the DataItem.
   * Note that based on the returned type, VIPR will "cast" the DataItem to either ICatDataItem or IContDataItem.
   * @returns {string} The DataItemType, e.g. `"cat"` or `"cont"`.
   */


		DataItem.prototype.getType = function getType() {
			return this.type;
		};

		// IItemClass

		/**
   * Get the caption of the item class.
   * @param {string} _kind The FormatKind of caption to retrieve.
   * @returns {string} The caption of the item class.
   */


		DataItem.prototype.getCaption = function getCaption() /*_kind*/{
			return this.caption;
		};

		/**
   * Get the subcaption of the item class.
   * @param {string} _kind The FormatKind of subcaption to retrieve.
   * @returns {string|null} The caption of the item class, or `null` if not applicable.
   */


		DataItem.prototype.getSubCaption = function getSubCaption() /*_kind*/{
			return null;
		};

		/**
   * Get the unique name of the item class.
   * @returns {string} The unique name.
   */


		DataItem.prototype.getUniqueName = function getUniqueName() {
			return this.uniqueName;
		};

		return DataItem;
	}(Decorated);

	/**
  * Generic Array iterator to comply with VIPR's iterator based API for data points.
  */


	var DataPointIterator = function () {
		function DataPointIterator(_array) {
			_classCallCheck(this, DataPointIterator);

			// This is basically a "self destructing" iterator, as soon as it reaches the end it drops the reference to the array
			Object.defineProperties(this, {
				_i: {
					writable: true,
					value: -1
				},
				_a: {
					writable: true,
					value: _array
				}
			});
		}

		DataPointIterator.prototype.nextValue = function nextValue() {
			var result = null;
			if (this._a) {
				var i = ++this._i;

				if (i < this._a.length) result = this._a[i];else this._a = null;
			}
			return result;
		};

		return DataPointIterator;
	}();

	return {
		Decorated: Decorated,
		/*
  * A tuple consists of an ordered list of items. The number of items can be
  * derived from the number of item classes of the Set that the tuple belongs to.
  */
		Tuple: function (_Decorated2) {
			_inherits(Tuple, _Decorated2);

			function Tuple(_label, _id) {
				_classCallCheck(this, Tuple);

				var _this2 = _possibleConstructorReturn(this, _Decorated2.call(this));

				Object.defineProperties(_this2, {
					label: {
						enumerable: true,
						value: _label
					},
					id: {
						enumerable: true,
						value: _id
					}
				});
				return _this2;
			}

			/** Convenience method to make this object represent itself as String */


			Tuple.prototype.toString = function toString() {
				return this.label;
			};

			// ITuple

			/**
    * Get the index of the ItemClassSet this tuple belongs to.
    * Note: if the index is out of bounds (e.g. -1, or 0 with no sets), it is assumed the tuple doesn't belong to a set.
    * @returns {number} The index of the ItemClassSet.
    */


			Tuple.prototype.getItemClassSetIndex = function getItemClassSetIndex() {
				// No multi itemclass set support.
				return 0;
			};

			/**
    * Get the item at the specified item index. The number of items is defined by the number of ItemClasses of the set that the tuple belongs to.
    * @param {number} _index The item index.
    * @returns {com/ibm/vipr/data/IItem} The item.
    */


			Tuple.prototype.getItem = function getItem(_index) {
				// Shortcut: no stacking support, so simply return this.
				return _index === 0 ? this : null;
			};

			// IItem

			/**
    * Get the caption of the item class.
    * @param {string} _kind The FormatKind of caption to retrieve.
    * @returns {string} The caption of the item class.
    */


			Tuple.prototype.getCaption = function getCaption() /*_kind*/{
				// No support for different kinds
				return this.label;
			};

			/**
    * Get the unique name of the item class.
    * @returns {string} The unique name
    */


			Tuple.prototype.getUniqueName = function getUniqueName() {
				return this.id;
			};

			return Tuple;
		}(Decorated),

		/**
   * Scenario is a simple wrapper to contain the inputs, results and Tuple associated with a scenario.
   * It implements the Tuple interface so it can be used as a Tuple on a virtual categorical item describing the scenarios.
   */
		Scenario: function (_Decorated3) {
			_inherits(Scenario, _Decorated3);

			function Scenario(_caption, _id, _inputs, _results) {
				_classCallCheck(this, Scenario);

				var _this3 = _possibleConstructorReturn(this, _Decorated3.call(this));

				Object.defineProperties(_this3, {
					caption: {
						enumerable: true,
						writable: true,
						value: _caption
					},
					id: {
						enumerable: true,
						writable: true,
						value: _id
					},
					inputs: {
						enumerable: true,
						writable: true,
						value: _inputs
					},
					results: {
						enumerable: true,
						writable: true,
						value: _results
					}
				});
				return _this3;
			}

			Scenario.prototype.update = function update(_caption, _inputs, _results) {
				this.caption = _caption;
				this.inputs = _inputs;
				this.results = _results;
				return this;
			};

			/**
    * Get the index of the ItemClassSet this tuple belongs to.
    * Note: if the index is out of bounds (e.g. -1, or 0 with no sets), it is assumed the tuple doesn't belong to a set.
    * @returns {number} The index of the ItemClassSet.
    */


			Scenario.prototype.getItemClassSetIndex = function getItemClassSetIndex() {
				// No multi itemclass set support.
				return 0;
			};

			/**
    * Get the item at the specified item index. The number of items is defined by the number of ItemClasses of the set that the tuple belongs to.
    * @param {number} _index The item index.
    * @returns {com/ibm/vipr/data/IItem} The item.
    */


			Scenario.prototype.getItem = function getItem(_index) {
				// Shortcut: no stacking support, so simply return this.
				return _index === 0 ? this : null;
			};

			// IItem

			/**
    * Get the caption of the item class.
    * @param {string} _kind The FormatKind of caption to retrieve.
    * @returns {string} The caption of the item class.
    */


			Scenario.prototype.getCaption = function getCaption() /*_kind*/{
				// No support for different kinds
				return this.caption;
			};

			/**
    * Get the unique name of the item class.
    * @returns {string} The unique name
    */


			Scenario.prototype.getUniqueName = function getUniqueName() {
				return this.id;
			};

			return Scenario;
		}(Decorated),
		ContDataItem: function (_DataItem) {
			_inherits(ContDataItem, _DataItem);

			function ContDataItem(_caption, _uniqueName, _kind, _columnIndex, _formatter, _aggregation) {
				_classCallCheck(this, ContDataItem);

				var _this4 = _possibleConstructorReturn(this, _DataItem.call(this, 'cont', _caption, _uniqueName, _kind, _columnIndex));

				Object.defineProperties(_this4, {
					formatter: {
						enumerable: true,
						value: _formatter
					},
					aggregation: {
						enumerable: true,
						value: _aggregation
					},
					_hasAggregation: {
						writable: true,
						value: false
					}
				});
				return _this4;
			}

			ContDataItem.prototype.setAggregationState = function setAggregationState(_hasAggregation) {
				this._hasAggregation = !!_hasAggregation;
			};

			// IItemClass

			ContDataItem.prototype.getSubCaption = function getSubCaption() /*_kind*/{
				if (this._hasAggregation) {
					switch (this.aggregation) {
						case 'avg':
							return msg_average;
						case 'sum':
							return msg_sum;
					}
				}
				return _DataItem.prototype.getSubCaption.call(this);
			};

			// IContDataItem

			/**
    * Get the ItemClass to use as a header for this DataItem
    * @returns {com/ibm/vipr/data/IItemClass} The ItemClass defining this DataItem.
    */


			ContDataItem.prototype.getItemClass = function getItemClass() {
				// Shortcut: implemented by this object
				return this;
			};

			/**
    * Get a formatter for the specified FormatKind, to format generated values, for example values rendered on an axis label.
    * @param {String} The {@link com/ibm/vipr/data/FormatKind FormatKind} to get a Formatter for, e.g. FormatKind.Label (recommended for axis labels) or FormatKind.Data.
    * @returns {com/ibm/vipr/data/IFormatter}
    */


			ContDataItem.prototype.getFormatter = function getFormatter() /*_kind*/{
				// No FormatKind support, Dashboard only has a single formatter.
				return this.formatter;
			};

			/**
    * Get the domain of the data.
    * @param {com/ibm/vipr/api/ICapability} _aggregation The aggregation capability, or null if no aggregation is required.
    *      This aggregation should always match what was returned by getCapabilities (e.g. null if there was no aggregation capability, or same parameters if there was).
    * @returns {com/ibm/vipr/data/IContDomain|null} The min and max of the domain, or null to auto-determine from the data.
    */


			ContDataItem.prototype.getDomain = function getDomain() /*_aggregation, _channel*/{
				// Returning null means VIPR will calculate the domain. That's OK since all rows are available.
				return null;
			};

			return ContDataItem;
		}(DataItem),

		CatDataItem: function (_DataItem2) {
			_inherits(CatDataItem, _DataItem2);

			function CatDataItem(_caption, _uniqueName, _kind, _columnIndex, _tuples) {
				_classCallCheck(this, CatDataItem);

				var _this5 = _possibleConstructorReturn(this, _DataItem2.call(this, 'cat', _caption, _uniqueName, _kind, _columnIndex));

				Object.defineProperty(_this5, 'tuples', {
					enumerable: true,
					value: _tuples
				});
				return _this5;
			}

			// ICatDataItem

			/**
    * The number of ItemClassSets on the DataItem.
    * @returns {number}
    */


			CatDataItem.prototype.getItemClassSetCount = function getItemClassSetCount() {
				// Not supported; item class sets allow assymetrical query results.
				return 1;
			};

			/**
    * Get the set at the specified index.
    * @param {number} _index The zero-based index of the ItemClassSet to retrieve.
    * @returns {com/ibm/vipr/data/IItemClassSet} The ItemClassSet.
    */


			CatDataItem.prototype.getItemClassSet = function getItemClassSet(_index) {
				// Not supported, simply return this
				return _index === 0 ? this : null;
			};

			/**
    * The total number of tuples on the DataItem. This should indicate the total number of tuples on an DataItem, **not** the
    * number of Tuples in the current data window. When iterating Tuples, VIPR will respect the
    * {@link com/ibm/vipr/data/ICatDataItem#getDataWindowStart getDataWindowStart()} and {@link com/ibm/vipr/data/ICatDataItem#getDataWindowEnd getDataWindowEnd()} values.
    * @returns {Number}
    */


			CatDataItem.prototype.getTupleCount = function getTupleCount() {
				return this.tuples.length;
			};

			/**
    * The data window start of the DataItem.
    * The start index is the index of the first tuple that can be retrieved on this DataItem.
    * @returns {number}
    */


			CatDataItem.prototype.getDataWindowStart = function getDataWindowStart() {
				// No windowing support
				return 0;
			};

			/**
    * The data window end of the DataItem. Defined as a tuple range from start (inclusive) to end (exclusive).
    * The end index is the first out of bounds index (so end - start is the amount of tuples in the data window).
    * @returns {number}
    */


			CatDataItem.prototype.getDataWindowEnd = function getDataWindowEnd() {
				// No windowing support
				return this.tuples.length;
			};

			/**
    * Get a tuple at the specified tuple index.
    * VIPR will not call getTuple with an index that is outside of the data window.
    * @param {number} _indexOnDataItem The index of the tuple, relative to the absolute start of the ICatDataItem.
    * @returns {Tuple|undefined} The tuple at the specified index.
    */


			CatDataItem.prototype.getTuple = function getTuple(_indexOnDataItem) {
				return this.tuples[_indexOnDataItem];
			};

			// IItemClassSet

			/**
    * The number of item classes of this set. The item classes of a set basically define the
    * dimensionality of a set of tuples.
    * @returns {number}
    */


			CatDataItem.prototype.getItemClassCount = function getItemClassCount() {
				// No stacking support for this implementation, so return 1
				return 1;
			};

			/**
    * Get the item class at the specified index.
    * @param {number} _index The index of the item class.
    * @returns {com/ibm/vipr/data/IItemClass} The item class.
    */


			CatDataItem.prototype.getItemClass = function getItemClass(_index) {
				// Anotther shortcut; no stacking is supported so return this object itself.
				return _index === 0 ? this : null;
			};

			return CatDataItem;
		}(DataItem),

		Value: function (_Decorated4) {
			_inherits(Value, _Decorated4);

			function Value(_dataItem) {
				_classCallCheck(this, Value);

				var _this6 = _possibleConstructorReturn(this, _Decorated4.call(this));

				Object.defineProperties(_this6, {
					_di: {
						value: _dataItem
					},
					// Value (summed)
					_v: {
						writable: true,
						value: 0
					},
					// Count; number of values added
					_c: {
						writable: true,
						value: 0
					}
				});
				return _this6;
			}

			/**
    * Add a value to the Value object.
    * @param {number|null} _value
    */


			Value.prototype.add = function add(_value) {
				if (typeof _value === 'number' && !Number.isNaN(_value)) {
					this._v += _value;
					this._c++;
				}
			};

			/**
    * The type of value.
    * @returns {string} A vipr ValueType.
    */


			Value.prototype.getValueType = function getValueType() {
				return this._c < 1 ? 'missing' : 'numeric';
			};

			/**
    * Get the value as a formatted string. This method should always return a correct value to
    * display as a String, for example an empty string for Missing.
    * @param {String} _kind ({@link com/ibm/vipr/data/FormatKind FormatKind}) The kind of format.
    * @returns {String} The formatted value.
    */


			Value.prototype.getCaption = function getCaption() /*_kind*/{
				return this._di.formatter.format(this.getValue());
			};

			/**
    * Gets the underlying value in its native type.
    * @member {Number|String|Date}
    */


			Value.prototype.getValue = function getValue() {
				if (this._c < 1) return null;
				switch (this._di.aggregation) {
					case 'sum':
						return this._v;
					case 'avg':
						return this._v / this._c;
				}
			};

			return Value;
		}(Decorated),

		DataPoint: function (_Decorated5) {
			_inherits(DataPoint, _Decorated5);

			function DataPoint(_dataItems, _rowData, _offset) {
				_classCallCheck(this, DataPoint);

				var _this7 = _possibleConstructorReturn(this, _Decorated5.call(this));

				Object.defineProperties(_this7, {
					dataItems: {
						enumerable: true,
						value: _dataItems
					},
					_rowData: {
						value: _rowData
					},
					_offset: {
						value: _offset
					}
				});
				return _this7;
			}

			/**
    * Validate the data item index, since going out of bounds may actually put us in a different data point.
    * @param {number} _dataItemIndex
    * @returns {number} A valid index.
    */


			DataPoint.prototype._validateIndex = function _validateIndex(_dataItemIndex) {
				if (_dataItemIndex < 0 || _dataItemIndex >= this.dataItems.length) {
					throw new RangeError('Data item index out of range');
				}
				return _dataItemIndex;
			};

			/**
    * Get the index of the tuple for the specified DataItem.
    * Will only be called for indices that are reported to have a categorical data item.
    * @param {number} _dataItemIndex The index of DataItem to get the tuple index for.
    * @returns {number} The index of the Tuple on the associated CatDataItem.
    */


			DataPoint.prototype.getTupleIndex = function getTupleIndex(_dataItemIndex) {
				return this._rowData[this._offset + this._validateIndex(_dataItemIndex)];
			};

			/**
    * Get the value for the specified DataItem.
    * Will only be called for indices that are reported to have a continuous data item.
    * @param {Number} _dataItemIndex The index of DataItem to get the value for.
    * @returns {com/ibm/vipr/data/IValue} The value at the specified index (for the associated {@link com/ibm/vipr/data/IContDataItem IContDataItem}),
    * or null if there is no value.
    */


			DataPoint.prototype.getValue = function getValue(_dataItemIndex) {
				return this._rowData[this._offset + this._validateIndex(_dataItemIndex)];
			};

			return DataPoint;
		}(Decorated),

		/**
   * A DataSet is VIPR's name for a query result. The view is a hybrid between tabular and multi-dimensional data.
   * "Columns" are called DataItems, which can be categorical (discrete) or continuous (scalar).
   * This class also implements IDataSetMapping, since it's easy to combine the mapping with the data.
   */
		VIPRDataSet: function (_Decorated6) {
			_inherits(VIPRDataSet, _Decorated6);

			function VIPRDataSet(_dataItems, _dataPoints, _slots, _mapping) {
				_classCallCheck(this, VIPRDataSet);

				var _this8 = _possibleConstructorReturn(this, _Decorated6.call(this));

				Object.defineProperties(_this8, {
					dataItems: {
						enumerable: true,
						value: _dataItems
					},
					dataPoints: {
						enumerable: true,
						value: _dataPoints
					},
					slots: {
						enumerable: true,
						value: _slots
					},
					mapping: {
						enumerable: true,
						value: _mapping
					}
				});
				return _this8;
			}

			// IDataSet

			/**
    * Get the number of DataItems in the data set.
    * @returns {Number} The number of DataItems in the data.
    */


			VIPRDataSet.prototype.getDataItemCount = function getDataItemCount() {
				return this.dataItems.length;
			};

			/**
    * Get the DataItem at the specified edge index.
    * @param {Number} _index The index of the data item to get.
    * @returns {com/ibm/vipr/data/IDataItem} The data item at the specified index.
    */


			VIPRDataSet.prototype.getDataItem = function getDataItem(_index) {
				return this.dataItems[_index];
			};

			/**
    * Get the data point iterator.
    * @returns {com/ibm/vipr/data/IDataPointIterator} The data point iterator.
    */


			VIPRDataSet.prototype.getDataPointIterator = function getDataPointIterator() {
				return new DataPointIterator(this.dataPoints);
			};

			// IDataSetMapping

			/**
    * Get the indices of data items mapped to the specified slot.
    * Return an empty array to not map any items (`null` is also allowed).
    * @param {com/ibm/vipr/vizdef/SlotRef} _slotRef The slot reference definition.
    * @returns {number[]|null} The array of DataItem indices to map, or `null` (equivalent of empty array)
    */


			VIPRDataSet.prototype.getDataItemsFor = function getDataItemsFor(_slotRef) {
				var slotIndex = this.slots.indexOf(_slotRef);
				return slotIndex < 0 ? null : this.mapping[slotIndex];
			};

			return VIPRDataSet;
		}(Decorated)
	};
});
//# sourceMappingURL=Decorated.js.map