'use strict';

define(function () {
	var MAX_DRIVERS = 5;

	/**
  * Generic Ajax request using ajaxSvc.
  * @param ajaxSvc The Ajax service
  * @param {string} url
  * @param {object} data
  * @return {Promise} - (response)
  */
	function request(ajaxSvc, url, data, type) {
		return ajaxSvc.ajax({
			type: type,
			url: url,
			contentType: 'application/json',
			data: data,
			xhrFields: {
				withCredentials: true
			}
		});
	}

	// return the object that defines this module
	return {
		getKeyDrivers: function getKeyDrivers(ajaxSvc, module, targetFieldIDForExpression) {
			var _this = this;

			var sourceType = 'module'; //renderInfo.visView.visModel.getModule().getSourceType()
			var modID = module.getAssetId();
			var tempModule = module.getTemporarySessionModuleJSON();
			var tempModuleId = tempModule.use[0];
			var tempModuleStr = JSON.stringify(tempModule);
			var shapedModule = !!module.getShapingId();

			return this.getRecommendations(ajaxSvc, modID, shapedModule, targetFieldIDForExpression, tempModuleStr, tempModuleId).then(function (input) {
				var keyDriverSpec = JSON.stringify({
					'version': '1.1',
					'targetId': targetFieldIDForExpression,
					'inputs': input,
					'filters': [],
					'dataFilters': [],
					'methods': ['drivers']
				});
				return _this.fastPatternKeyDrivers(ajaxSvc, sourceType, 'false', 'none', keyDriverSpec, tempModuleId).then(function (drivers) {
					return drivers.slice(0, MAX_DRIVERS);
				});
			});
		},

		/**
   * Get recommendations columns
   *
   * @param  {string} modID
   * @param  {string} targetFieldIDForExpression
   * @param  {string} tempModule
   * @param  {string} tempModuleId
   * @return {promise} - recommendations
   */
		fastPatternKeyDrivers: function fastPatternKeyDrivers(ajaxSvc, type, refreshmd, qfb, keyDriverSpec, tempModuleId) {
			var path = 'v1/datasets/' + tempModuleId + '/fastpatternkeydrivers';
			return request(ajaxSvc, path, {
				'type': type,
				'refreshmd': refreshmd,
				'qfb': qfb,
				'moduleUrl': false,
				'keyDriverSpec': keyDriverSpec
			}, 'GET').then(function (response) {
				var keyDrivers = [];
				var labels = [];
				response.data.results.map(function (res, idx) {
					res.fields.inputs.map(function (input) {
						// add everything in the first group
						if (idx === 0) {
							keyDrivers.push(input.id);
							labels.push(input.label);
						} else {
							if (!(keyDrivers.indexOf(input.id) !== -1) && !(labels.indexOf(input.label) !== -1)) {
								keyDrivers.push(input.id);
								labels.push(input.label);
							}
						}
					});
				});
				return keyDrivers;
			});
		},

		/**
   * Get recommendations columns
   *
   * @param  {string} modID
   * @param  {string} targetFieldIDForExpression
   * @param  {string} tempModule
   * @param  {string} tempModuleId
   * @return {promise} - recommendations
   */
		getRecommendations: function getRecommendations(ajaxSvc, modID, shapedModule, targetFieldIDForExpression, tempModule, tempModuleId) {
			return request(ajaxSvc, 'v1/predict/recommendations', JSON.stringify({
				'moduleId': modID,
				'shapedModule': shapedModule,
				'targetFieldIDForExpression': targetFieldIDForExpression,
				'tempModule': tempModule,
				'tempModuleId': tempModuleId,
				'tuningParams': {
					'allowCategoryTarget': true,
					'noCorrelationLimit': true
				}
			}), 'POST').then(function (response) {
				var fieldIDForExpression = [];

				response.data.results.recommendedFields.fields.map(function (field) {
					fieldIDForExpression.push(field.fieldIDForExpression);
				});

				return fieldIDForExpression;
			});
		}

	};
});
//# sourceMappingURL=KeyDrivers.js.map