'use strict';

var _createClass = function () { function defineProperties(target, props) { for (var i = 0; i < props.length; i++) { var descriptor = props[i]; descriptor.enumerable = descriptor.enumerable || false; descriptor.configurable = true; if ("value" in descriptor) descriptor.writable = true; Object.defineProperty(target, descriptor.key, descriptor); } } return function (Constructor, protoProps, staticProps) { if (protoProps) defineProperties(Constructor.prototype, protoProps); if (staticProps) defineProperties(Constructor, staticProps); return Constructor; }; }();

function _classCallCheck(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

/**
 * Licensed Materials - Property of IBM
 * IBM Cognos Products: BI Cloud (C) Copyright IBM Corp. 2020
 * US Government Users Restricted Rights - Use, duplication or disclosure restricted by GSA ADP Schedule Contract with IBM Corp.
 */
define(['com/ibm/vida/vipr', 'com/ibm/vida/controls/vida-controls', 'com/ibm/vida/utils/vida-utils', './Column', './Decorated', './ScenarioUtils'], function (vipr, vidacontrols, vidautils, Column, Decorated, ScenarioUtils) {
	var bundles = Object.freeze([
	// Area
	{
		id: 'area',
		location: 'com/ibm/vida/vizbundles/rave2bundlearea'
	},
	// Bar
	{
		id: 'bar',
		location: 'com/ibm/vida/vizbundles/rave2bundlebar'
	},
	// Bubble
	{
		id: 'bubble',
		location: 'com/ibm/vida/vizbundles/rave2bubble'
	},
	// Composite
	{
		id: 'composite',
		location: 'com/ibm/vida/vizbundles/rave2bundlecomposite'
	},
	// Line
	{
		id: 'line',
		location: 'com/ibm/vida/vizbundles/rave2line'
	},
	// Pie
	{
		id: 'pie',
		location: 'com/ibm/vida/vizbundles/rave2bundlepie'
	}]);
	/**
  * * VIPR acts as a static repository; web applications should only ccreate a single instance.
  * * The used vida widget (SmallMultipleGrid) will load the visualization bundle using this vipr instance.
  * */
	function _getVIPR() {
		return vipr.VIPR.create({
			bundles: bundles
		});
	}

	function _hasTooltip() {
		// TODO: implement tooltip
		return false;
	}

	function _showTooltip(_point, _items) {
		// TODO: implement tooltip
		// Note: the _point is in local coordinates, relative to the visualization top-left.
		// To translate to client coordinates, use getBoundingClientRect() on the visualization node and add the
		// left, top values to the x, y values respectively.
		if (typeof console !== 'undefined') {
			console.debug('show tooltip for', _items);
		}
	}

	function _updateTooltip() /*_point, _items*/{
		// TODO: implement tooltip
	}

	function _hideTooltip() {}
	// TODO: implement tooltip


	/** Caption of the virtual Row Column & DataItem. */
	var msg_row_column = 'Row';
	/** Used to create a caption for each virtual Row Tuple. */
	var msg_row_tuple = '{0}';

	/*
  * Added a dependency to the vida-utils StringEx utility class since it provides a useful format function.
  * Dashboard/LiveWidget may provide an alternative utility that may be a better alternative, if you want to avoid
  * the dependency on Vida's utilities.
  */
	var StringEx = vidautils.StringEx;

	/**
  * The what-if extensions takes a query result, which would normally be drawn in a visualization, as input data.
  * This means that the query result, to this extensions, is source data. An AI model can be trained upon it and
  * that model can be used to predict values. These predictions may be given to a visualization, which requires
  * a DataSet too. This is why this module encodes two types of data sets: a source table and visualization data.
  */

	/**
  * RowsColumn is a virtual column to encode a virtual Row Number categorcal.
  * Mapping this virtual column will lead to a categorical data item that has each row number as separate tuple.s
  */
	var RowsColumn = new Column.CategoricalColumn(msg_row_column, '(row#)', 'rows', null);

	/**
  * ColumnMapping allows mapping Column instances to slots. This cannot be visualized directly, a processing step
  * needs to happen to turn a Table + ColumnMapping into VIPRData + SlotMapping. This is performed in createVIPRData.
  */

	var ColumnMapping = function () {
		function ColumnMapping(_vizDef) {
			_classCallCheck(this, ColumnMapping);

			Object.defineProperties(this, {
				vizDef: {
					enumerable: true,
					value: _vizDef
				},
				_aggregate: {
					writable: true,
					value: false
				},
				_mapping: {
					value: new Array(_vizDef.slots.length).fill(null)
				}
			});
		}

		/**
   * Get whether aggregation is preferred or not.
   * Note that setting this to `false` for visualization that require aggregation has no effect.
   */


		ColumnMapping.prototype.add = function add(_slotName, _column) {
			var slot = this.vizDef.getSlot(_slotName);
			if (!slot) {
				throw new Error('There is no ' + _slotName + ' slot');
			}
			var slotIndex = this.vizDef.slots.indexOf(slot);

			var mapping = this._mapping[slotIndex];

			if (_column) {
				if (!Column.isColumn(_column)) {
					throw new TypeError('_column must be a Column');
				}
				if (!mapping) {
					mapping = [];
					this._mapping[slotIndex] = mapping;
				}
				// Set/add the column
				if (mapping.indexOf(_column) < 0) {
					if (slot.multiDataItem) {
						mapping.push(_column);
					} else {
						mapping[0] = _column;
					}
				}
			} else if (mapping) {
				// Clear existing mapping
				this._mapping[slotIndex] = null;
			}
		};

		ColumnMapping.prototype.get = function get(_slotName) {
			var slotIndex = this.vizDef.slots.indexOf(this.vizDef.getSlot(_slotName));
			return slotIndex < 0 ? null : this._mapping[slotIndex];
		};

		_createClass(ColumnMapping, [{
			key: 'aggregate',
			get: function get() {
				return this._aggregate;
			},
			set: function set(_value) {
				this._aggregate = !!_value;
			}
		}]);

		return ColumnMapping;
	}();

	/**
  * VIPRData is the root class to provide data to VIPR. It's basically a placeholder for DataSets, which is what
  * VIPR calls a "query result". This object also implements ISlotMapping, so it can be passed as slot mapping too.
  */


	var VIPRData = function () {
		function VIPRData(_dataSets) {
			_classCallCheck(this, VIPRData);

			Object.defineProperties(this, {
				dataSets: {
					enumerable: true,
					value: _dataSets
				}
			});
		}

		// IVIPRData

		/**
   * Get the data set at the specified index.
   * @param {number} _index The index of the data set.
   * @returns {VIPRDataSet} The data set.
   */


		VIPRData.prototype.getDataSetAt = function getDataSetAt(_index) {
			return this.dataSets[_index];
		};

		/**
   * Get the version of the decorations. If the decorations on VIPR data are updated, change
   * (e.g. increment) the returned version number to notify VIPR of this. Doing so does not
   * require a setData call, it will be picked up the next time decorations are requested.
   * @returns {number} A number representing the version of decorations.
   */


		VIPRData.prototype.getDecorationsVersion = function getDecorationsVersion() {
			return 0; // Not yet supported.
		};

		// ISlotMapping

		/**
   * Get the slot mapping for the data set at the specified index.
   * @param {number} _index The index of the data set.
   * @returns {VIPRDataSet} The data set mapping.
   */


		VIPRData.prototype.getDataSetMapping = function getDataSetMapping(_index) {
			// Just like VIPRData implements ISlotMapping, VIPRDataSet implements IDataSetMapping.
			return this.dataSets[_index];
		};

		return VIPRData;
	}();

	/**
  * Helper method to find the index of a DataItem that maps to the specified `_columnIndex`.
  * @param {DataItem[]} _dataItems The array of data items to search.
  * @param {number} _columnIndex The index of the desired referenced column
  * @returns {number} The resolved index, or `-1` if no data item matched.
  */


	function findDataItemIndex(_dataItems, _columnIndex) {
		for (var i = 0; i < _dataItems.length; i++) {
			if (_dataItems[i].columnIndex === _columnIndex) {
				return i;
			}
		}
		return -1;
	}

	/**
  * Create a DataItem from a Column
  * @param {Column} _column The column this DataItem is based on.
  * @param {number} _columnIndex The index of the column this data item is based on.
  * @returns {DataItem} The data item.
  */
	function createDataItem(_column, _columnIndex) {
		if (_column.type === 'measure') {
			return new Decorated.ContDataItem(_column.label, _column.id, _column.kind, _columnIndex, _column.formatter, ScenarioUtils.toSupportedAggregation(_column.aggregation));
		} else /*if (_column.type === 'categorical')*/{
				return new Decorated.CatDataItem(_column.label, _column.id, _column.kind, _columnIndex,
				// Copy tuples if they are not frozen (should only be for the Scenario)
				Object.isFrozen(_column.tuples) ? _column.tuples : Object.freeze(_column.tuples.slice(0)));
			}
	}

	/**
  * Create a virtual Rows DataItem for the specified rowCount
  * @param {number} _rowCount The row count to generate tuples for.
  * @returns {CatDataItem} The virtual data item.
  */
	function createRowsDataItem(_rowCount) {
		var tuples = new Array(_rowCount);
		for (var i = 0; i < _rowCount; i++) {
			tuples[i] = new Decorated.Tuple(StringEx.format(msg_row_tuple, i), 'row#' + i);
		}
		return new Decorated.CatDataItem(RowsColumn.label, RowsColumn.id, RowsColumn.kind, -1, Object.freeze(tuples));
	}

	/**
  * Query a visualization data set, aggregating over unmapped categoricals as needed
  * @param {DataSetDef} _dataSetDef The VIPR DataSet definition.
  * @param {DeltaTable} _table The table to iterate for data.
  * @param {ColumnMapping} _columnMapping The column
  */
	function queryDataSet(_dataSetDef, _table, _columnMapping) {
		/** The mapping array contains the actual slot mapping. */
		var mapping = new Array(_dataSetDef.slots.length).fill(null);
		/** The data items as presented to the visualization */
		var dataItems = [];
		// Track if there is a virtual Rows DataItem.
		var rowsDataItem = null;
		_dataSetDef.slots.forEach(function (_slotRef, _index) {
			var columnMapping = _columnMapping.get(_slotRef.name);
			if (columnMapping) {
				/*
     * The column mapping contains Column instances.
     * VIPR wants DataItem indices in the mapping.
     * Translate the mapping to data item indices, creating a DataItem for a column if it doesn't exist.
     */
				mapping[_index] = columnMapping.map(function (_column) {
					var columnIndex = void 0;
					if (_column === RowsColumn) {
						// Use index -1 to encode the virtual rows column
						columnIndex = -1;
					} else {
						columnIndex = _table.columns.indexOf(_column);
						// In this case the -1 value is invalid, it should not default to the virtual rows column.
						if (columnIndex < 0) {
							// Program error, this should not happen.
							throw new Error('Mapped column is not part of the provided table');
						}
					}
					// Check if the column is already mapped to a data item
					var dataItemIndex = findDataItemIndex(dataItems, columnIndex);
					// If not, map a new data item
					if (dataItemIndex < 0) {
						var dataItem = void 0;
						if (columnIndex >= 0) {
							dataItem = createDataItem(_column, columnIndex);
						} else if (columnIndex === -1) {
							dataItem = createRowsDataItem(_table.baseRowCount);
							rowsDataItem = dataItem;
						}
						// Add the data item, and update the resolved index for the mapping.
						dataItemIndex = dataItems.push(dataItem) - 1;
					}
					return dataItemIndex;
				});
			}
		});

		// This step is optional: add unmapped continuous items to the data points too. This makes it easier to show them (aggregated) in a tooltip.
		for (var columnIndex = 0; columnIndex < _table.columns.length; columnIndex++) {
			var column = _table.columns[columnIndex];
			if (column.type === 'measure' && findDataItemIndex(dataItems, columnIndex) < 0) {
				// Add the unmapped cont data item
				dataItems.push(createDataItem(column, columnIndex));
			}
		}

		Object.freeze(dataItems);

		var volume = 1;
		var catItems = [];
		var ordinalOffsets = [];
		for (var dataItemIndex = 0; dataItemIndex < dataItems.length; dataItemIndex++) {
			var dataItem = dataItems[dataItemIndex];
			if (dataItem.type === 'cat' && dataItem.tuples.length !== 1) {
				catItems.push(dataItem);
				ordinalOffsets.push(volume);
				volume *= dataItem.tuples.length;
			}
		}

		/**
   * Many visualizations require aggregation, encoded by the uniqueDataPoints flag on the DataSetDef.
   * If visualizations support un-aggregated data points, check the preference set on the ColumnMapping.
   *
   * In this DeltaTable, there is one combination in which we know aggregation can never occur:
   * if both the virtual row and scenario columns are mapped, each DataPoint will be unique.
   *
   * This effectively means this flag can become `false` even though the visualization requires aggregation.
   */
		var shouldAggregate = (_dataSetDef.uniqueDataPoints || _columnMapping.aggregate) && ( // Must aggregate (by definition)?
		!rowsDataItem || findDataItemIndex(dataItems, 0) < 0); // Is aggregation even "possible"?
		/**
   * Maps ordinal to DataPoint. This is a map since the space may be sparse, e.g. have many missing rows.
   * If this cache is `null`, it means logic determined no aggregation was needed.
   */
		var dataPointCache = shouldAggregate ? new Map() : null;
		// If there is no aggregation, the amount of data points is known. In that case, initialize with the correct capacity.
		var dataPoints = dataPointCache ? new Array(_table.baseRowCount) : [];
		dataPoints.length = 0;
		// Allocate data. If there is aa rowsDataItem the amount of rows is known exactly.
		// If not, start with a capacity for 16 rows and let the array grow as needed.
		var rowData = new Array(dataItems.length * (dataPointCache ? _table.baseRowCount : 16));
		rowData.length = 0;

		/** Flag that tracks wether at least one value is aggregated. (If not, the aggregation captions can be avoided). */
		var hasAggregatedData = false;

		_table.iterate(function (_row, _columns, _rowIndex, _baseRowIndex) {
			// Start by calculating the ordinal for this row
			var ordinal = void 0;
			if (dataPointCache) {
				// Calculate the ordinal based on the mapped categorical items.
				ordinal = 0;
				for (var i = 0; i < catItems.length; ++i) {
					var tupleIndex = catItems[i] === rowsDataItem ? _baseRowIndex : _row[catItems[i].columnIndex];
					ordinal += tupleIndex * ordinalOffsets[i];
				}
			} else {
				// If there is no need to aggregate, simply use the row index of the delta table as ordinal.
				ordinal = _rowIndex;
			}

			var dataPoint = dataPointCache && dataPointCache.get(ordinal);
			if (!dataPoint) {
				// Create a new DataPoint
				var dataPointOffset = rowData.length;
				for (var _dataItemIndex = 0; _dataItemIndex < dataItems.length; _dataItemIndex++) {
					var _dataItem = dataItems[_dataItemIndex];
					if (_dataItem.type === 'cont') {
						rowData.push(new Decorated.Value(_dataItem));
					} else if (_dataItem === rowsDataItem) {
						// The tuple index is the source row index
						rowData.push(_baseRowIndex);
					} else {
						// Simply use the tuple index
						rowData.push(_row[_dataItem.columnIndex]);
					}
				}
				dataPoint = new Decorated.DataPoint(dataItems, rowData, dataPointOffset);
				dataPoints.push(dataPoint);
				// If there is no aggregation, there is no point in putting the DataPoint in a cache;
				// the ordinal will never be hit again.
				if (dataPointCache) {
					dataPointCache.set(ordinal, dataPoint);
				}
			} else if (!hasAggregatedData) {
				// Adding values to an existing data point: the data points contain aggregated data
				hasAggregatedData = true;
			}

			for (var _dataItemIndex2 = 0; _dataItemIndex2 < dataItems.length; ++_dataItemIndex2) {
				var _dataItem2 = dataItems[_dataItemIndex2];
				if (_dataItem2.type === 'cont') {
					dataPoint.getValue(_dataItemIndex2).add(_row[_dataItem2.columnIndex]);
				}
			}
		});

		for (var _dataItemIndex3 = 0; _dataItemIndex3 < dataItems.length; ++_dataItemIndex3) {
			var _dataItem3 = dataItems[_dataItemIndex3];
			if (_dataItem3.type === 'cont') {
				_dataItem3.setAggregationState(hasAggregatedData);
			}
		}

		return new Decorated.VIPRDataSet(dataItems, dataPoints, _dataSetDef.slots, mapping);
	}
	/**
  * Create a ColumnMapping object, that allows mapping columns to slots. This mapping can be used to run a
  * "query" on a table to build visualization data.
  * @param {VizDef} _vizDef The VizBundle vizDef.
  * @returns {ColumnMapping} The ColumnMapping that allows mapping columns to slots..
  */
	function createColumnMapping(_vizDef) {
		return new ColumnMapping(_vizDef);
	}

	/**
  * Create VIPR (visualization) data from a DeltaTable and slot mapping
  * @param {DeltaTable} _table The table to iterate.s
  * @param {ColumnMapping} _columnMapping The column mapping to apply.
  * Unmapped categoricals will be removed, aggregating values.
  */
	function createViprData(_table, _columnMapping) {
		var dataSets = _columnMapping.vizDef.dataSets.map(function (_dataSetDef) {
			return queryDataSet(_dataSetDef, _table, _columnMapping);
		});
		return new VIPRData(dataSets);
	}

	return {
		/*
  * Added a dependency to the vida-utils StringEx utility class since it provides useful format and join functions.
  * Dashboard/LiveWidget may provide an alternative utility that may be a better alternative, if you want to avoid
  * the dependency on Vida's utilities.
  */
		getStringEx: function getStringEx() {
			return vidautils.StringEx;
		},

		// Visualization
		getVidaWidget: function getVidaWidget(style) {
			return new vidacontrols.SmallMultipleGrid({
				vipr: _getVIPR(),
				style: style ? style : 'width:100%;height:100%'
			});
		},
		getActionHandler: function getActionHandler(vidaWidget) {
			return new vidacontrols.LegacyActionHandler.GestureToAction(vidaWidget, {
				hasTooltip: _hasTooltip.bind(this),
				showTooltip: _showTooltip.bind(this),
				updateTooltip: _updateTooltip.bind(this),
				hideTooltip: _hideTooltip.bind(this)
			});
		},

		RowsColumn: RowsColumn,

		/**
   * Create a slot mapping for the visualization and create data for it.
   * @param {VizDef} _vizDef The Vida visualization definition.
   */
		mapDataToViz: function mapDataToViz(_vizDef, deltaTable) {
			var columns = deltaTable.columns;
			var mapping = createColumnMapping(_vizDef);
			var targetIndex = columns.findIndex(Column.isTargetColumn);

			/*
    * For now, use a hard coded slot mapping for the Bar's slots:
    * categories (x-axis)
    * values (y-axis)
    * yStart (not used)
    * target (not used)
    * color (legend items)
    */

			// Note that if both the Scenario and virtual Row column are mapped, dat will never be aggregated.

			// Always map the scenario column to the color slot
			mapping.add('color', columns[0]);
			// Map the predicted target to the values slot
			mapping.add('values', columns[targetIndex]);
			// If context is not provided, aggreagte the data

			// See if there is any categorical context column left to show as categories.
			var categoricalContextColumn = columns.find(function (_column) {
				return _column.type === 'categorical' && Column.isContextColumn(_column);
			});
			// If not, map the virtual "rows" column so data is not aggregated.
			mapping.add('categories', categoricalContextColumn);
			return createViprData(deltaTable, mapping);
		}
	};
});
//# sourceMappingURL=VidaUtils.js.map